/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    gclamp                                                    *
*Project Version: 1.2.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 17th May 2011                                     *
*                                                                           *
*Start DateStamp: 04-May-2009 10.15.10 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:   http://balean.net/linprogs                                         *
*                                                                           *
*Project Summary:                                                           *
* Gtk++ frontend for CLAMav Program                                         *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       view.cpp                                                  *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       30-Dec-2010 14.39.30 -36000 3600                          *
*                                                                           *
*File Summary:                                                              *
* Module to view freshclam.log and gclamp.log                               *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include <fstream>

#include "view.h"

View::~View(void)
{
//cout << "~View:\n";
if (pixbuf) g_object_unref(G_OBJECT(pixbuf));
// pango_attribute_destroy(attr); // It apears that "pango_attr_list_unref(attr_list);" does this for us
pango_attr_list_unref(attr_list);
}


View::View(Gclamp *g, string title, string text)
{	
//cout << "View:\n";
this->g = g;
this->title = title;
pixbuf = NULL;
create_display(text);
}



void View::close_dialog (GtkWidget *widget, View *view)
{

if (view->title=="freshclam.log")
  {
  view->g->view_freshclam_log = NULL;
  }
else if (view->title=="gclamp.log")
  {
  view->g->view_gclamp_log = NULL;
  }
else if (view->title=="gclamp.defaults")
  {
  view->g->view_gclamp_defaults = NULL;
  }
else view->g->view_about = NULL;
delete view; 
}

void View::quit_view(GtkWidget *widget, View *view)
{
gtk_widget_destroy ( GTK_WIDGET(view->window) );
}

void View::callback_clear_gclamp_log(GtkWidget *widget, View *view)
{
string logname = view->g->home_directory + "/gclamp.log";
unlink(logname.c_str());
gtk_widget_destroy ( GTK_WIDGET(view->window) );
}

void View::create_display(string text)
{ 
  GtkWidget *scrolled_window;
  GtkWidget *label;
  GtkWidget *box, *vbox;
  GtkWidget *image;
  GtkWidget *button;

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_show (window);
  g_signal_connect (G_OBJECT (window), "destroy",
		    G_CALLBACK (close_dialog),
		    this);

  gtk_window_set_title (GTK_WINDOW (window), title.c_str());
  gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER_ALWAYS);
//  gtk_widget_set_size_request (window, g->width, g->height);
  gtk_window_set_keep_above(GTK_WINDOW(window), TRUE);

  gtk_container_set_border_width(GTK_CONTAINER(window), 2);
  pixbuf = gtk_widget_render_icon(GTK_WIDGET(window),
				  GTK_STOCK_ABOUT,
				  GTK_ICON_SIZE_DIALOG,
				  "V" );
  if (pixbuf) gtk_window_set_icon( GTK_WINDOW(window), pixbuf);


  vbox = gtk_vbox_new(FALSE, 10);
  gtk_container_add (GTK_CONTAINER (window), vbox);


  scrolled_window = gtk_scrolled_window_new(NULL, NULL);

    /* the policy is one of GTK_POLICY AUTOMATIC, or GTK_POLICY_ALWAYS.
     * GTK_POLICY_AUTOMATIC will automatically decide whether you need
     * scrollbars, whereas GTK_POLICY_ALWAYS will always leave the scrollbars
     * there.  The first one is the horizontal scrollbar, the second, 
     * the vertical. */
  gtk_scrolled_window_set_policy ( GTK_SCROLLED_WINDOW(scrolled_window),
                                GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS );

  gtk_widget_show (scrolled_window);
  gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, TRUE, TRUE, 0);

  box = gtk_vbox_new(FALSE, 0);
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window),
					box);

  image = gtk_image_new();
  gtk_misc_set_alignment(GTK_MISC (image), 0, .5); // left
  gtk_image_set_from_pixbuf(GTK_IMAGE(image), g->icon_pixbuf);
  gtk_box_pack_start (GTK_BOX(box), image, FALSE, TRUE, 0);

  attr = pango_attr_family_new("monospace");
  
  attr_list = pango_attr_list_new();
  pango_attr_list_insert(attr_list, attr);

  label = gtk_label_new(text.c_str());
  gtk_misc_set_alignment(GTK_MISC (label), 0, .5); // left
  gtk_label_set_attributes(GTK_LABEL(label), attr_list);
  gtk_box_pack_start (GTK_BOX(box), label, FALSE, TRUE, 0);


  box = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_end (GTK_BOX(vbox), box, FALSE, FALSE, 0);



  // --- Create the "Close" button
  button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
  gtk_widget_show(GTK_WIDGET(gtk_button_get_image(GTK_BUTTON(button)))); 

  // --- Need to close the window if they press "Close"
  g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK ( quit_view ),  this);

  // --- Allow it to be the default button
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);

  // --- Add the button to the dialog
  gtk_box_pack_end (GTK_BOX(box), button, FALSE, TRUE, 0);

  // --- Make the button the default button
  gtk_widget_grab_default (button);

  // --- Create the "Clear" button
  if (title == "gclamp.log")
    {
	button = gtk_button_new_from_stock(GTK_STOCK_CLEAR);
	gtk_widget_show(GTK_WIDGET(gtk_button_get_image(GTK_BUTTON(button))));
	g_signal_connect (G_OBJECT (button), "clicked",
		    G_CALLBACK ( callback_clear_gclamp_log ),  this);
	gtk_box_pack_end (GTK_BOX(box), button, FALSE, TRUE, 0);
    }

  // Set the window size
  GtkRequisition ri, rl, rb;
  gint w, h;
  gtk_widget_size_request(GTK_WIDGET(image), &ri);
  gtk_widget_size_request(GTK_WIDGET(label), &rl);
  gtk_widget_size_request(GTK_WIDGET(button), &rb);

  w = (rl.width+40 > g->width) ? g->width : rl.width+40;
  h = (ri.height+rl.height+rb.height+20> g->height) ? g->height : ri.height+rl.height+rb.height+20;

//  cout << "VIEW: - w= " << w << " : h= " << h << endl;
  
  gtk_widget_set_size_request (window, w, h);

  // --- Make everything visible
  gtk_widget_show_all(window);

}
