/****************************************************************************
*RevHead_Start                                                              *
*                                                                           *
* Project Details                                                           *
* ---------------                                                           *
*Project Name:    gclamp                                                    *
*Project Version: 1.2.0                                                     *
*Copyright Owner: David M. Balean <linprogs@balean.net>                     *
*Copyright Notice:                                                          *
* This program is free software; you can redistribute it and/or modify it   *
* under the terms of the GNU General Public License as published by         *
* the Free Software Foundation, either version 3 of the License or,         *
* (at your option) any later version.                                       *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                      *
* See the GNU General Public License for more details.                      *
* You should have received a copy of the GNU General Public License         *
* along with this program. If not, see <http://www.gnu.org/licenses/>.      *
*                                                                           *
*Project Date:    Tuesday 17th May 2011                                     *
*                                                                           *
*Start DateStamp: 04-May-2009 10.15.10 -36000 0                             *
*                                                                           *
*Contact:                                                                   *
* email: linprogs@balean.net                                                *
* snail: D.M. Balean, 44 Wyong Road, Killarney Vale, N.S.W. 2261, AUSTRALIA *
* URL:   http://balean.net/linprogs                                         *
*                                                                           *
*Project Summary:                                                           *
* Gtk++ frontend for CLAMav Program                                         *
*                                                                           *
*****************************************************************************
*                                                                           *
*This File:       update.cpp                                                *
*Author:          David M. Balean <linprogs@balean.net>                     *
*DateStamp:       08-Feb-2011 16.51.54 -36000 3600                          *
*                                                                           *
*RevHead_End                                                                *
****************************************************************************/

#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <pwd.h>
#include <grp.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>

#include "../common/common.h"
#include "../common/vtesu.h"

#include "update.h"

using namespace common;

string Update::log_pathname = "/var/log/freshclam.log";
guint Update::visible_signal;


struct Freshclam_Codes Update::codes[17] =
  {
  0, "Database successfully updated",
  1, "Database is up-to-date",
  40, "Unknown option passed",
  50, "Can’t change directory",
  51, "Can’t check MD5 sum",
  52, "Connection (network) problem",
  53, "Can’t unlink file",
  54, "MD5 or digital signature verification error",
  55, "Error reading file",
  56, "Config file error",
  57, "Can’t create new file",
  58, "Can’t read database from remote server",
  59, "Mirrors are not fully synchronized (try again later)",
  60, "Can’t get information about ’’ user from /etc/passwd",
  61, "Can’t drop privileges",
  62, "Can’t initialize logger",
  100, "UNKNOWN ERROR"
  };





Update::~Update(void)
{
if (pixbuf) g_object_unref(G_OBJECT(pixbuf));
}

void Update::callback_quit(GtkWidget *widget, Update *up)
{
if (up->fd >= 0)
  {
  close(up->fd);
  }
exit(0);
}

void Update::callback_close(GtkWidget *widget, Update *up)
{
gtk_widget_destroy(up->window);
}

void Update::send_message(string message, Update *up)
{
GtkWidget *wid;
char buffer[PIPE_BUF];

int len = sprintf(buffer, message.c_str());
  
if ((write(up->fd, buffer, len+1)) < 0)
  {
  // ERROR
  message = "FAILURE: Update write to gclamp error: ";
  message += strerror(errno);
  wid = gtk_label_new(message.c_str());
  gtk_misc_set_alignment(GTK_MISC (wid), 0, .5); // left
  gtk_box_pack_start(GTK_BOX(up->box), wid, FALSE, FALSE, 0);
  }
}

gboolean Update::do_update(gpointer gdata)
{
string message;

Update *up = (Update *)gdata;
int rc = up->new_process("freshclam");

if (rc < 0)
  {
  // new_process for freshclam failed
  message = "FAILURE: new process for freshclam failed";
  up->send_message(message, up);
  }
  
else
  {
  struct Freshclam_Codes *code;
  int n;
  for (n=0; n<16; ++n)
    {
    code = &codes[n];
    if (rc == code->return_code)
      {
      break;
      }
    }
  message = codes[n].message; // codes[16] is "UNKNOWN ERROR"
  
  if (n>1) message = "Freshclam FAILURE: " + message;
  up->send_message(message, up);

  }
// sleep(8); // Test the close button - supposed to appear when finished
up->finished = TRUE;
return TRUE;
}



// Difficult to tell if this actually does what I want
void Update::callback_visible(GtkWidget *widget, GdkEvent *event, Update *up)
{
//gtk_main_iteration_do(FALSE); // This runs a single iteration of the main loop
  
gtk_signal_disconnect(GTK_OBJECT(widget), visible_signal);
if (up->error)
  {
  return;
  }
// Try a thread_create

gtk_widget_hide(up->close_button);
gtk_main_iteration_do(FALSE); // This runs a single iteration of the main loop

pthread_t thread;
up->finished = FALSE;
if ( pthread_create(
                   &thread,
                   NULL,
                   (void *(*)(void *))do_update,
                   (void *)up
                   ) )
  {
  cerr << "New thread for update failed" << endl;
  gtk_label_set_label(GTK_LABEL(up->update_label), "New thread for update failed");
  gtk_main_iteration_do(FALSE); // This runs a single iteration of the main loop
  }
  
else
  {
  int i=0;
  bool label_red = FALSE;
    
  gtk_label_set_text(GTK_LABEL(up->update_label), "UPDATING");
  gtk_widget_modify_fg(GTK_WIDGET(up->update_label), GTK_STATE_NORMAL, &black);
    
  for (;;)
    {
    if (up->finished) break;
    if ((i) == 1000)
      {
      if (label_red)
        {
        gtk_widget_modify_fg(GTK_WIDGET(up->update_label), GTK_STATE_NORMAL, &black);
        label_red = FALSE;
        }
      else
        {
        gtk_widget_modify_fg(GTK_WIDGET(up->update_label), GTK_STATE_NORMAL, &red);
        label_red = TRUE;   
        }
      i=0;
      }
    else i++;

    gtk_main_iteration_do(FALSE); // This runs a single iteration of the main loop
    usleep(200);
    }
  }

gtk_label_set_text(GTK_LABEL(up->update_label), "FINISHED");
gtk_widget_modify_fg(GTK_WIDGET(up->update_label), GTK_STATE_NORMAL, &blue);
gtk_widget_show(up->close_button);
gtk_widget_show(GTK_WIDGET(gtk_button_get_image(GTK_BUTTON(up->close_button)))); 
gtk_main_iteration_do(FALSE); // This runs a single iteration of the main loop
}


// If we get here and not root, restart as root using VTESU

Update::Update(int argc, char *argv[])
{
GtkWidget *wid, *hbox;
string message;
string command;
int rc;
struct stat64 statbuf;
struct passwd *pwd;
struct group *clamav_group;  


fd = -1;
error = FALSE;
pixbuf = NULL;

// cout << "Update::Update" << endl;

// Check if root
// If not, restart as root
sleep(1);
gtk_init (&argc, &argv);

if ( (strcmp( (getpwuid( getuid() ))->pw_name , "root" )) ) // if we are not root
  {


//  cout << "Update::Update - argv[0]= \"" << argv[0] << "\"" << endl;

	char *array[5] = // { NULL };
			/* */
			{
			argv[0],
			argv[0],
			(char *)"-e0",
			(char *)"-ni",
			(char *)NULL
			};



	Vtesu vsu(4, array);

//	cout << "Update::Update - got here" << endl;

	sleep(1);  // Give it time to start...
	return;  // quick exit...
  }


// Open a window here
// This needs to be on top of everything else
// and near the centre of the screen - fix later
window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
pixbuf = gtk_widget_render_icon(GTK_WIDGET(window),
				  GTK_STOCK_ABOUT,
				  GTK_ICON_SIZE_DIALOG,
				  "V" );
if (pixbuf) gtk_window_set_icon( GTK_WINDOW(window), pixbuf);

gtk_window_set_title(GTK_WINDOW(window), "gclamp_udpate");
g_signal_connect (G_OBJECT (window), "destroy",
		    G_CALLBACK (callback_quit),
		    this);
  

visible_signal = g_signal_connect (G_OBJECT (window), "window_state_event", //"visibility_event",
  G_CALLBACK (callback_visible),
  this);

  
gtk_widget_set_size_request (window, 400, 300);   // width, height
gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER_ALWAYS);
gtk_window_set_keep_above(GTK_WINDOW(window), TRUE);

box = gtk_vbox_new(TRUE, 2);
gtk_container_add(GTK_CONTAINER(window), box);
  

fd = open(fifo_name.c_str(), O_WRONLY);
if (fd < 0)
  {
  // ERROR
  message = "Update: open fifo error: ";
  message += strerror(errno);
  wid = gtk_label_new(message.c_str());
  gtk_misc_set_alignment(GTK_MISC (wid), 0, .5); // left
  gtk_box_pack_start(GTK_BOX(box), wid, FALSE, FALSE, 0);
  error = TRUE;
  goto SHOW;
  }
else
  {
  wid = gtk_label_new("Update: open fifo SUCCESS");
  gtk_misc_set_alignment(GTK_MISC (wid), 0, .5); // left
  gtk_box_pack_start(GTK_BOX(box), wid, FALSE, FALSE, 0);
  }

if ( (strcmp( (getpwuid( getuid() ))->pw_name , "root" )) ) // if we are not root
  {
  message = "FAILURE: Update must be run by root!";
  wid = gtk_label_new(message.c_str());
  gtk_misc_set_alignment(GTK_MISC (wid), 0, .5); // left
  gtk_box_pack_start(GTK_BOX(box), wid, FALSE, FALSE, 0);

  send_message(message, this);

  error = TRUE;
  goto SHOW;
  }
  
wid = gtk_label_new("Update: executed by root");
gtk_misc_set_alignment(GTK_MISC (wid), 0, .5); // left
gtk_box_pack_start(GTK_BOX(box), wid, FALSE, FALSE, 0);


if (!internet_connected())
  {
  message = "FAILURE: no internet connection";
  wid = gtk_label_new(message.c_str());
  gtk_misc_set_alignment(GTK_MISC (wid), 0, .5); // left
  gtk_box_pack_start(GTK_BOX(box), wid, FALSE, FALSE, 0);

  send_message(message, this);

  error = TRUE;
  goto SHOW;
  }

wid = gtk_label_new("Update: internet available");
gtk_misc_set_alignment(GTK_MISC (wid), 0, .5); // left
gtk_box_pack_start(GTK_BOX(box), wid, FALSE, FALSE, 0);
  

// Now check we have /var/log/freshclam.log
// If not, create it
if (!isfile(log_pathname.c_str(), NULL))
  {
  message = "Update: creating " + log_pathname;
  wid = gtk_label_new(message.c_str());
  gtk_misc_set_alignment(GTK_MISC (wid), 0, .5); // left
  gtk_box_pack_start(GTK_BOX(box), wid, FALSE, FALSE, 0);

  if ( (rc = new_process(command = "touch " + log_pathname)) )
    {
    message = "FAILURE: command \"" + command + "\"";
    wid = gtk_label_new(message.c_str());
    gtk_misc_set_alignment(GTK_MISC (wid), 0, .5); // left
    gtk_box_pack_start(GTK_BOX(box), wid, FALSE, FALSE, 0);

    send_message(message, this);

    error = TRUE;
    goto SHOW;
    }
  }
// Check ownership
// Owner & Group should be root:clamav

if (lstat64(log_pathname.c_str(), &statbuf))
  {
  message = "FAILURE: lstat64 \"" + log_pathname + "\": ";
  message += strerror(errno);
  
  wid = gtk_label_new(message.c_str());
  gtk_misc_set_alignment(GTK_MISC (wid), 0, .5); // left
  gtk_box_pack_start(GTK_BOX(box), wid, FALSE, FALSE, 0);

  send_message(message, this);

  error = TRUE;
  goto SHOW;
  }
message = "Update: lstat64 of \"" + log_pathname + "\" OK";
wid = gtk_label_new(message.c_str());
gtk_misc_set_alignment(GTK_MISC (wid), 0, .5); // left
gtk_box_pack_start(GTK_BOX(box), wid, FALSE, FALSE, 0);

pwd = getpwuid(statbuf.st_uid);
if ((pwd) && (pwd->pw_name != "root"))
  {
  gid_t gid = 0;  // default is root
    
  message = "Update : \"" + log_pathname + "\" NOT owned by root";
  wid = gtk_label_new(message.c_str());
  gtk_misc_set_alignment(GTK_MISC (wid), 0, .5); // left
  gtk_box_pack_start(GTK_BOX(box), wid, FALSE, FALSE, 0);


  if ( (clamav_group = getgrnam("clamav")) )
    {
    gid = clamav_group->gr_gid;
    }

  if ( (chown(log_pathname.c_str(), 0, gid)) ) // should return 0 if OK
    {
    message = "FAILURE: chown \"" + log_pathname + "\" to root:clamav: ";
    message += strerror(errno);

    wid = gtk_label_new(message.c_str());
    gtk_misc_set_alignment(GTK_MISC (wid), 0, .5); // left
    gtk_box_pack_start(GTK_BOX(box), wid, FALSE, FALSE, 0);

    send_message(message, this);

    error = TRUE;
    goto SHOW;
    }

  message = "Update : \"" + log_pathname + "\" now owned by root";
  wid = gtk_label_new(message.c_str());
  gtk_misc_set_alignment(GTK_MISC (wid), 0, .5); // left
  gtk_box_pack_start(GTK_BOX(box), wid, FALSE, FALSE, 0);

  }

// Check permissions
if (statbuf.st_mode != 33206) // i.e. 0666
  {
  message = "Update : \"" + log_pathname + "\" permissions not 0666";
  wid = gtk_label_new(message.c_str());
  gtk_misc_set_alignment(GTK_MISC (wid), 0, .5); // left
  gtk_box_pack_start(GTK_BOX(box), wid, FALSE, FALSE, 0);

  if ( (chmod(log_pathname.c_str(), 0666)) ) // should return 0 if OK
    {
    message = "FAILURE: chmod\"" + log_pathname + "\" to 0666: ";
    message += strerror(errno);

    wid = gtk_label_new(message.c_str());
    gtk_misc_set_alignment(GTK_MISC (wid), 0, .5); // left
    gtk_box_pack_start(GTK_BOX(box), wid, FALSE, FALSE, 0);

    send_message(message, this);

    error = TRUE;
    goto SHOW;
    }
  message = "Update : \"" + log_pathname + "\" permissions changed to 0666";
  wid = gtk_label_new(message.c_str());
  gtk_misc_set_alignment(GTK_MISC (wid), 0, .5); // left
  gtk_box_pack_start(GTK_BOX(box), wid, FALSE, FALSE, 0);

  }

update_label = gtk_label_new("");
gtk_misc_set_alignment(GTK_MISC (wid), 0, .5); // left
gtk_box_pack_start(GTK_BOX(box), update_label, FALSE, FALSE, 0);


SHOW:
hbox = gtk_hbox_new(FALSE, 5);
gtk_box_pack_end(GTK_BOX(box), hbox, FALSE, FALSE, 0);
close_button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
gtk_widget_show(GTK_WIDGET(gtk_button_get_image(GTK_BUTTON(close_button)))); 
gtk_box_pack_end(GTK_BOX(hbox), close_button, FALSE, FALSE, 0);
g_signal_connect (G_OBJECT (close_button), "clicked",
		    G_CALLBACK (callback_close),
		    this);
gtk_widget_show_all(GTK_WIDGET(window) );
gtk_main();
}

struct  addrinfo *Update::get_ip(char *address_name)
{
struct addrinfo hints, *res;
char ipstr[INET6_ADDRSTRLEN];
int status;

memset(&hints, 0, sizeof hints);
hints.ai_family = AF_UNSPEC; // AF_INET or AF_INET6 to force version
hints.ai_socktype = SOCK_STREAM;

if (getaddrinfo(address_name, "http", &hints, &res) != 0)
	{
  res = NULL;
  }

return res;
}




bool Update::internet_connected(void)
{
struct addrinfo *p, *res;
char ipstr[INET6_ADDRSTRLEN];
void *addr;
char *ipver;
int sockfd;
bool connected = FALSE;

static char *address_name[] = 
  {
  (char *)"www.microsoft.com", 
  (char *)"www.google.com",
  (char *)"www.redhat.com"
  };

for (int n=0; ((n<3) && (!connected)); ++n)
  {
  res = get_ip(address_name[n]);
  if (res)
	  {
	  for (p=res; p!= NULL; p=p->ai_next)
		  {
		  // get the pointer to the address itself,
		  // different fields in IPv4 and IPv6:
		  if (p->ai_family == AF_INET)
			  {
			  // IPv4
    	  struct sockaddr_in *ipv4 = (struct sockaddr_in *)p->ai_addr;
    	  addr = &(ipv4->sin_addr);
    	  ipver = (char *)"IPv4";
    	  }
  	  else
			  {
			  // IPv6
    	  struct sockaddr_in6 *ipv6 = (struct sockaddr_in6 *)p->ai_addr;
   		  addr = &(ipv6->sin6_addr);
   		  ipver = (char *)"IPv6";
   		  }

		  // convert the IP to a string and print it:
  	  inet_ntop(p->ai_family, addr, ipstr, sizeof ipstr);

  	
  	
  	  // make a socket
  	  sockfd = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
		  if (sockfd == -1)
			  {
			  }
		  else
			  {
			  if (connect(sockfd, res->ai_addr, res->ai_addrlen) == -1)
				  {
				  }
			  else
				  {
          connected = TRUE;
				  }
			  }
		  close(sockfd);
  	  }

	  freeaddrinfo(res); // free the linked list
	  }
  }
  

return connected;
} 







// GENERAL PURPOSE VERSION
int Update::new_process(string command)
{
pid_t pid;

if (!command.length()) return(-2);   // NOTE: -1 is returned as error by many shell progs, so we start at -2
if ((pid = fork()) == -1) return(-3);

//==============
// CHILD PROCESS
//==============
if (pid == 0)
  {
  char *argv[4];
  argv[0] = (char *)"sh";
  argv[1] = (char *)"-c";
  argv[2] = (char *)command.c_str();
  argv[3] = 0;

//  cout << "CHILD PROCESS" << endl;

  execve("/bin/sh", argv, environ);

  exit(-4);                          // Error if it returns...
  }
//===============
// PARENT PROCESS
//===============
//cout << "PARENT PROCESS" << endl;

for (;;)                                 // Parent process
  {
  int status = 0;

  if (waitpid(pid, &status, 0) == -1)
    {
	  if (!(errno == EINTR)) return(-5);
    }
  else if (WIFEXITED(status))
    {
	  status = (WEXITSTATUS(status));

	  return(status);   // All normal exit codes are >= 0
    }
  else
    {
	  return(-6);
    }
  }  
}


// Does file exist? - Optionally with size of file in bytes
bool Update::isfile(string pathname, off_t *file_size)
{
struct stat64 statbuf;

if (!(lstat64(pathname.c_str(), &statbuf)))
  {
  if (S_ISREG(statbuf.st_mode))
    {
    if (file_size)  *file_size = statbuf.st_size;
    return(TRUE);
    }
  }
return(FALSE); // Not a regular file or ERROR
}



